@@ -25,20 +25,12 @@
 #include <openobex/obex.h>
 
 #include "obexserver.h"
+#include "obex-glib.h"
 
 #define OBEX_PUSH_HANDLE	10
 
 #define _(x)  (x)
 
-struct chan
-{
-  GIOChannel *chan;
-  int fd;
-  int source;
-};
-
-static GList *channels;
-
 static void
 file_received (gchar *name, const uint8_t *data, size_t data_len)
 {
@@ -126,48 +118,6 @@ handle_request (obex_t *handle, obex_object_t *object, int event, int cmd)
     }
 }
 
-static void
-handle_disconnect (obex_t *handle)
-{
-  int fd;
-  GList *i;
-
-  fd = OBEX_GetFD (handle);
-
-  for (i = channels; i; i = i->next)
-    {
-      struct chan *chan = i->data;
-
-      chan = i->data;
-
-      if (chan->fd == fd)
-	{
-	  channels = g_list_remove (channels, chan);
-
-	  g_source_remove (chan->source);
-
-	  g_io_channel_unref (chan->chan);
-
-	  g_free (chan);
-
-	  return;
-	}
-    }
-
-  fprintf (stderr, "Couldn't find channel in list.\n");
-  abort ();
-}
-
-static gboolean
-io_callback (GIOChannel *source, GIOCondition cond, gpointer data)
-{
-  obex_t *obex = (obex_t *)data;
-
-  OBEX_HandleInput (obex, 0);
-
-  return TRUE;
-}
-
 void
 obex_conn_event (obex_t *handle, obex_object_t *object, int mode, int event, int obex_cmd, int obex_rsp)
 {
@@ -193,26 +143,21 @@ obex_conn_event (obex_t *handle, obex_object_t *object, int mode, int event, int
       break;
 
     case OBEX_EV_LINKERR:
-      handle_disconnect (handle);
+      obex_disconnect_input (handle);
       break;
     }
 }
 
-void
+static void
 obex_event (obex_t *handle, obex_object_t *object, int mode, int event, int obex_cmd, int obex_rsp)
 {
   obex_t *obex;
-  struct chan *chan;
 
   switch (event)
     {
     case OBEX_EV_ACCEPTHINT:
-      chan = g_malloc (sizeof (*chan));
       obex = OBEX_ServerAccept (handle, obex_conn_event, NULL);
-      chan->fd = OBEX_GetFD (obex);
-      chan->chan = g_io_channel_unix_new (chan->fd);
-      chan->source = g_io_add_watch (chan->chan, G_IO_IN | G_IO_ERR | G_IO_HUP, io_callback, obex);
-      channels = g_list_prepend (channels, chan);
+      obex_connect_input (obex);
       break;
 
     default:
@@ -235,6 +180,7 @@ add_opush (sdp_session_t *session, uint8_t chan)
   int i;
   uint8_t dtd = SDP_UINT8;
   sdp_data_t *sflist;
+  gboolean ret = TRUE;
   
   memset((void *)&record, 0, sizeof(sdp_record_t));
   record.handle = 0xffffffff;
@@ -267,12 +213,6 @@ add_opush (sdp_session_t *session, uint8_t chan)
   
   aproto = sdp_list_append(0, apseq);
   sdp_set_access_protos(&record, aproto);
-  sdp_data_free(channel);
-  sdp_list_free(proto[0], 0);
-  sdp_list_free(proto[1], 0);
-  sdp_list_free(proto[2], 0);
-  sdp_list_free(apseq, 0);
-  sdp_list_free(aproto, 0);
   
   for (i = 0; i < sizeof(formats); i++) 
     {
@@ -284,12 +224,17 @@ add_opush (sdp_session_t *session, uint8_t chan)
   
   sdp_set_info_attr(&record, "OBEX Object Push", 0, 0);
   
-  if (0 > sdp_record_register(session, &record, SDP_RECORD_PERSIST)) 
-    {
-      printf("Service Record registration failed.\n");
-      return FALSE;
-    }
-  return TRUE;
+  if (sdp_record_register(session, &record, SDP_RECORD_PERSIST) < 0)
+    ret = FALSE;
+
+  sdp_data_free(channel);
+  sdp_list_free(proto[0], 0);
+  sdp_list_free(proto[1], 0);
+  sdp_list_free(proto[2], 0);
+  sdp_list_free(apseq, 0);
+  sdp_list_free(aproto, 0);
+
+  return ret;
 }
 
 int 
@@ -297,20 +242,17 @@ add_service (void)
 {
   sdp_session_t *sess;
   bdaddr_t interface;
+  gboolean ret;
 
   sess = sdp_connect (&interface, BDADDR_LOCAL, 0);
 
   if (!sess)
     return FALSE;
 
-  if (add_opush (sess, 10) == FALSE)
-    {
-      sdp_close(sess);
-      return FALSE;
-    }
+  ret = add_opush (sess, 10);
 
   sdp_close(sess);
-  return TRUE;
+  return ret;
 }
 
 int
@@ -324,12 +266,14 @@ obex_init (void)
     return FALSE;
 
   BtOBEX_ServerRegister (obex, NULL, OBEX_PUSH_HANDLE);
+
+  obex_connect_input (obex);
   
-  chan = g_io_channel_unix_new (OBEX_GetFD (obex));
-
-  g_io_add_watch (chan, G_IO_IN | G_IO_ERR | G_IO_HUP, io_callback, obex);
-
-  add_service ();
+  if (add_service () == FALSE)
+    {
+      gpe_error_box (_("Couldn't register OBEX service"));
+      return FALSE;
+    }
 
   return TRUE;
 }
